# -*- coding: utf-8 -*-
"""
本模块功能：上市公司的财务报表分析，数据层
所属工具包：证券投资分析工具SIAT 
SIAT：Security Investment Analysis Tool
创建日期：2020年9月10日
最新修订日期：2020年9月14日
作者：王德宏 (WANG Dehong, Peter)
作者单位：北京外国语大学国际商学院
版权所有：王德宏
用途限制：仅限研究与教学使用，不可商用！商用需要额外授权。
特别声明：作者不对使用本工具进行证券投资导致的任何损益负责！
"""

#==============================================================================
#关闭所有警告
import warnings; warnings.filterwarnings('ignore')

import pandas as pd
#==============================================================================
#本模块使用yahooquery插件
#==============================================================================
if __name__=='__main__':
    symbol='INTL' 
    symbol='MSFT'
    symbol='600519.SS'

def get_balance_sheet(symbol):
    """
    功能：获取雅虎财经上一只股票所有的年度和季度资产负债表
    """
    print("Searching for balance info of",symbol,"\b, please wait...")
    
    from yahooquery import Ticker
    stock = Ticker(symbol)    
    
    import time
    #获取年报
    try:
        stmta=stock.balance_sheet()  # Defaults to Annual
    except:
        print("  Searching for annual report failed, recovering......")
        time.sleep(5)
        try:
            stmta=stock.balance_sheet()
        except:
            print("  #Warning(get_balance_sheet): no annual info available for",symbol)
            #return None
    """
    #判断是否抓取到了数据
    import pandas as pd
    if not isinstance(stmta,pd.DataFrame):
        print("\n  #Error(get_balance_sheet): no annual info available for",symbol)
        return None
    """
    #获取季度报
    try:
        stmtq=stock.balance_sheet(frequency="q")
    except:
        print("  Searching for quarterly report failed, recovering......")
        time.sleep(5)
        try:
            stmtq=stock.balance_sheet(frequency="q")
        except:
            print("  #Warning(get_balance_sheet): no quarterly info available for",symbol)
            #return None
    """
    if (len(stmta)==0) and (len(stmtq)==0):
        print("  #Error(get_balance_sheet): no reports available for",symbol)
        return None
    """
    #合并年度和季度报表
    if not (stmta is None) and not (stmtq is None):
        stmt=pd.concat([stmta,stmtq])
    elif not (stmta is None):
        stmt=stmta
    elif not (stmtq is None):
        stmt=stmtq        
    else:
        print("  #Error(get_balance_sheet): no reports available for",symbol)
        return None    
    
    #字段缺失替代处理
    #流动（有息）债务
    if 'CurrentDebt' not in list(stmt):
        stmt['CurrentDebt']=0.0
    #流动负债（有息债务+应付）
    if 'CurrentLiabilities' not in list(stmt):
        stmt['CurrentLiabilities']=stmt['CurrentDebt']
    #总（有息）债务
    if 'TotalDebt' not in list(stmt):
        stmt['TotalDebt']=0.0
    #少数股东权益
    if 'MinorityInterest' not in list(stmt):
        if 'TotalEquityGrossMinorityInterest' in list(stmt):
            stmt['MinorityInterest']= \
                stmt['TotalEquityGrossMinorityInterest']-stmt['StockholdersEquity']
        elif 'TotalLiabilitiesNetMinorityInterest' in list(stmt):
            stmt['MinorityInterest']=stmt['TotalAssets']- \
                stmt['TotalLiabilitiesNetMinorityInterest']- \
                    stmt['StockholersEquity']
        else:
            stmt['MinorityInterest']=0.0
    #总负债
    if 'TotalLiabilities' not in list(stmt): 
        if 'TotalLiabilitiesNetMinorityInterest' in list(stmt):
            stmt['TotalLiabilities']=stmt['TotalLiabilitiesNetMinorityInterest']+ \
                stmt['MinorityInterest']
        else:
            stmt['TotalLiabilities']=stmt['TotalDebt']+stmt['MinorityInterest']
    #权益总额
    if 'TotalEquities' not in list(stmt):
        stmt['TotalEquities']=stmt['TotalEquityGrossMinorityInterest']- \
            stmt['MinorityInterest']    
    #存货
    if 'Inventory' not in list(stmt):
        stmt['Inventory']=0.0
    #流动资产
    if 'CurrentAssets' not in list(stmt):
        stmt['CurrentAssets']=stmt['CashAndCashEquivalents']+ \
            stmt['Receivables']+stmt['Inventory']
    #应收账款
    if 'AccountsReceivable' not in list(stmt):
        stmt['AccountsReceivable']=stmt['Receivables']

    
    #总检查：总资产=总负债+总权益是否成立
    stmt['TA-TL-TE']=stmt['TotalAssets']-stmt['TotalLiabilities']- \
        stmt['TotalEquities']
    
    #排序
    stmt.sort_values(by=['asOfDate','periodType'],inplace=True)
    #去掉重复记录: 保留年报数据项多，去掉数据项少的季报
    stmt.drop_duplicates(subset=['asOfDate'],keep='first',inplace=True)

    #print(", done")
    return stmt    
    
if __name__ == '__main__':
    fs=get_balance_sheet('AAPL')
    fs_msft=get_balance_sheet('MSFT')
    fs_c=get_balance_sheet('C')
    fs_maotai=get_balance_sheet('600519.SS')

#==============================================================================
if __name__=='__main__':
    symbol='AAPL' 
    symbol='MST'

def get_income_statements(symbol):
    """
    功能：获取雅虎财经上一只股票所有的年度和季度利润表
    """
    print("Searching for income info of",symbol,"\b, please wait...")
    
    from yahooquery import Ticker
    stock = Ticker(symbol)    
    
    import time
    #获取年报
    try:
        stmta=stock.income_statement()  # Defaults to Annual
    except:
        print("  Searching for annual report failed, recovering......")
        time.sleep(5)
        try:
            stmta=stock.income_statement()
        except:
            print("  #Warning(get_income_statements): no annual info available for",symbol)
            #return None
    """
    #判断是否抓取到了数据
    import pandas as pd
    if not isinstance(stmta,pd.DataFrame):
        print("  #Error(get_income_statements): no annual info available for",symbol)
        return None
    """
    #获取季度报
    try:
        stmtq=stock.income_statement(frequency="q")
    except:
        print("Searching for quarterly report failed, recovering......")
        time.sleep(5)
        try:
            stmtq=stock.income_statement(frequency="q")
        except:
            print("  #Warning(get_income_statements): no quarterly info available for",symbol)
            #return None
    """
    if (len(stmta)==0) and (len(stmtq)==0):
        print("  #Error(get_income_statements): no reports available for",symbol)
        return None
    """
    #合并年度和季度报表
    if not (stmta is None) and not (stmtq is None):
        stmt=pd.concat([stmta,stmtq])
    elif not (stmta is None):
        stmt=stmta
    elif not (stmtq is None):
        stmt=stmtq        
    else:
        print("  #Error(get_income_statements): no reports available for",symbol)
        return None 
    
    #排序
    stmt.sort_values(by=['asOfDate','periodType'],inplace=True)
    #去掉重复记录: 保留年报数据项多，去掉数据项少的季报
    stmt.drop_duplicates(subset=['asOfDate'],keep='first',inplace=True)

    #字段缺失处理
    if 'InterestExpense' not in list(stmt):
        #import numpy as np
        #stmt['InterestExpense']=np.nan
        stmt['InterestExpense']=0
    
    # 服务型企业无此项
    if 'CostOfRevenue' not in list(stmt):
        stmt['CostOfRevenue']=0
    
    # 服务型企业
    if 'OperatingIncome' not in list(stmt):
        stmt['OperatingIncome']=stmt['OperatingRevenue']-stmt['OperatingExpense']
    
    # 服务型企业
    if 'GrossProfit' not in list(stmt):
        stmt['GrossProfit']=stmt['PretaxIncome']

    
    #print(", done")
    return stmt    
    
if __name__ == '__main__':
    stmt=get_income_statements('AAPL')
    stmt_msft=get_income_statements('MSFT')
    stmt_c=get_income_statements('C')
    stmt_maotai=get_income_statements('600519.SS')

#==============================================================================
if __name__=='__main__':
    symbol='AAPL' 

def get_cashflow_statements(symbol):
    """
    功能：获取雅虎财经上一只股票所有的年度和季度现金流量表
    """
    print("Searching for cashflow info of",symbol,"\b, please wait...")
    
    from yahooquery import Ticker
    stock = Ticker(symbol)    
    
    import time
    #获取年报
    try:
        stmta=stock.cash_flow()  # Defaults to Annual
    except:
        print("  Searching for annual report failed, recovering......")
        time.sleep(5)
        try:
            stmta=stock.cash_flow()
        except:
            print("  #Warning(get_cashflow_statements): no annual info available for",symbol)
            #return None
    """
    #判断是否抓取到了数据
    import pandas as pd
    if not isinstance(stmta,pd.DataFrame):
        print("\n  #Error(get_cashflow_statements): no annual info available for",symbol)
        return None
    """
    #获取季度报
    try:
        stmtq=stock.cash_flow(frequency="q")
    except:
        print("Searching for quarterly report failed, recovering......")
        time.sleep(5)
        try:
            stmtq=stock.cash_flow(frequency="q")
        except:
            print("  #Warning(get_cashflow_statements): no quarterly info available for",symbol)
            #return None
    """
    if (len(stmta)==0) and (len(stmtq)==0):
        print("  #Error(get_cashflow_statements): no reports available for",symbol)
        return None
    """
    #合并年度和季度报表
    if not (stmta is None) and not (stmtq is None):
        stmt=pd.concat([stmta,stmtq])
    elif not (stmta is None):
        stmt=stmta
    elif not (stmtq is None):
        stmt=stmtq        
    else:
        print("  #Error(get_cashflow_statements): no reports available for",symbol)
        return None 
    
    #排序
    stmt.sort_values(by=['asOfDate','periodType'],inplace=True)
    #去掉重复记录: 保留年报数据项多，去掉数据项少的季报
    stmt.drop_duplicates(subset=['asOfDate'],keep='first',inplace=True)

    #字段缺失处理
    if 'CashDividendsPaid' not in list(stmt):
        #import numpy as np
        #stmt['CashDividendsPaid']=np.nan
        stmt['CashDividendsPaid']=0

    #print(", done")
    return stmt    
    
if __name__ == '__main__':
    stmt=get_cashflow_statements('AAPL')
    stmt_msft=get_cashflow_statements('MSFT')
    stmt_c=get_cashflow_statements('C')
    stmt_maotai=get_cashflow_statements('600519.SS')

#==============================================================================
if __name__=='__main__':
    ticker='AAPL' 

def get_financial_statements(ticker):
    """
    功能：获取雅虎财经上一只股票所有的年度和季度财务报表
    """
    print("Searching for financial statements of",ticker,"\b, please wait")
    #获取资产负债表
    try:
        fbs = get_balance_sheet(ticker)
    except:
        print("  Retrieving fin info failed, trying to recover...")
        import time; time.sleep(3)
        try:
            fbs = get_balance_sheet(ticker)
        except:
            import time; time.sleep(5)
            try:
                fbs = get_balance_sheet(ticker)
            except:
                print("  #Error(get_financial_statements): balance sheet not available for",ticker)
                return None
    if fbs is None:
        print("  #Error(get_financial_statements): financial statements not available for",ticker)
        return None
    
    #获取利润表
    try:
        fis = get_income_statements(ticker)
    except:
        print("      Failed, recovering...")
        import time; time.sleep(3)
        try:
            fis = get_income_statements(ticker)
        except:
            import time; time.sleep(5)
            try:
                fis = get_income_statements(ticker)
            except:
                print("  #Error(get_financial_statements): income info not available for",ticker)
                return None
    if fis is None:
        print("  #Error(get_financial_statements): financial statements not available for",ticker)
        return None
    
    #获取现金流量表
    try:
        fcf = get_cashflow_statements(ticker)
    except:
        print("      Failed, recovering...")
        import time; time.sleep(3)
        try:
            fcf = get_cashflow_statements(ticker)
        except:
            import time; time.sleep(5)
            try:
                fcf = get_cashflow_statements(ticker)
            except:
                print("  #Error(get_financial_statements): cash flow info not available for",ticker)
                return None
    if fcf is None:
        print("  #Error(get_financial_statements): financial statements not available for",ticker)
        return None
    
    #合并1：资产负债表+利润表
    import pandas as pd
    fbs_fis=pd.merge(fbs,fis,on=['asOfDate','periodType'])
    
    #合并2：+现金流量表
    fbs_fis_fcf=pd.merge(fbs_fis,fcf,on=['asOfDate','periodType','NetIncome'])
    
    fbs_fis_fcf['ticker']=ticker
    #将所有的nan替换为0
    #fbs_fis_fcf.fillna(0,inplace=True)

    print("...Successfully retrieved financial statements of",ticker,"\b!")    
    return fbs_fis_fcf    
    
if __name__ == '__main__':
    fs=get_financial_statements('AAPL')
    fs_aapl=get_financial_statements('AAPL')
    fs_msft=get_financial_statements('MSFT')
    fs_c=get_financial_statements('C')
    fs_maotai=get_financial_statements('600519.SS')

"""
最终获得的表结构：
['asOfDate',
 'periodType',
 
 'AccountsPayable(应付账款)',
 'AccountsReceivable(应收账款)',
 'AccumulatedDepreciation（累计折旧）',
 'AdditionalPaidInCapital（资本公积，资本溢价，附加资本；paid-in capital：实收资本；缴入资本）',
 'AllowanceForDoubtfulAccountsReceivable（备抵应收呆帐）',
 'AvailableForSaleSecurities（可供出售金融资产；trading securities: 交易性金融资产）',
 'BuildingsAndImprovements（建筑物改良）',
 'CapitalStock（股本）',
 'CashAndCashEquivalents（现金及现金等价物）',
 'CashCashEquivalentsAndShortTermInvestments（现金、现金等价物及短期投资）',
 'CashEquivalents（现金等价物）',
 'CashFinancial（？）',
 'CommonStock（普通股）',
 'CommonStockEquity（普通股权益？）',
 'ConstructionInProgress（在建工程）',
 'CurrentAssets（流动资产）',
 'CurrentLiabilities（流动负债）',
 'DividendsPayable（应付股利）',
 'FinishedGoods（制成品）',
 'GoodwillAndOtherIntangibleAssets（商誉及其他无形资产）',
 'GrossAccountsReceivable（应收账款总额）',
 'GrossPPE（固定资产总额）',
 'InventoriesAdjustmentsAllowances（存货调整备抵）',
 'Inventory（存货）',
 'InvestedCapital（投入资本）',
 'InvestmentinFinancialAssets（金融资产投资？）',
 'LandAndImprovements（土地改良）',
 'MachineryFurnitureEquipment（机械家具设备？）',
 'MinorityInterest（少数股东损益？）',
 'NetPPE（固定资产净值）',
 'NetTangibleAssets（有形资产净值）',
 'NonCurrentDeferredAssets（非流动递延资产）',
 'NonCurrentDeferredTaxesAssets（非流动递延税项资产？）',
 'NonCurrentDeferredTaxesLiabilities（非流动递延税金负债？）',
 'OrdinarySharesNumber（普通股数量？）',
 'OtherCurrentAssets（其他流动资产）',
 'OtherCurrentLiabilities（其他流动负债）',
 'OtherEquityInterest（其他股权）',
 'OtherIntangibleAssets（其他有形资产）',
 'OtherNonCurrentAssets（其他非流动资产）',
 'OtherPayable（其它应付款）',
 'OtherProperties（？）',
 'OtherReceivables（其他应收款）',
 'Payables（应付款项）',
 'PrepaidAssets（预付资产；预付费用）',
 'Properties（财产？）',
 'RawMaterials（原材料）',
 'RetainedEarnings（留存收益）',
 'ShareIssued（股票发行）',
 'StockholdersEquity（股东权益）',
 'TangibleBookValue（有形资产账面价值）',
 'TotalAssets（总资产）',
 'TotalCapitalization（资本总额？）',
 'TotalEquityGrossMinorityInterest（少数股东权益总额）',
 'TotalLiabilitiesNetMinorityInterest（？）',
 'TotalNonCurrentAssets（非流动资产总额）',
 'TotalNonCurrentLiabilitiesNetMinorityInterest（？）',
 'TotalTaxPayable（应缴税款总额）',
 'TradeandOtherPayablesNonCurrent（？）',
 'WorkInProcess（在制品）',
 'WorkingCapital（营运资本）',
 'Amortization（摊销）',
 
 'BasicAverageShares（未稀释的平均股数？）',
 'BasicEPS（ 基本每股收益，指属于普通股股东的当期净利润，除以发行在外普通股的加权平均数，可按存在月数加权）',
 'CostOfRevenue（主营业务成本，营收成本）',
 'DepreciationAndAmortizationInIncomeStatement（损益表中的折旧和摊销）',
 'DepreciationIncomeStatement（损益表中的折旧）',
 'DilutedAverageShares（稀释后平均股数？）',
 'DilutedEPS（考虑了可转换债券和股票期权可能行权对于流通在外股数的影响）',
 'EBIT（息税前利润）',
 'EBITDA（未计利息、税项、折旧及摊销前的利润）',
 'GeneralAndAdministrativeExpense（一般管理费用）',
 'GrossProfit（营业毛利）',
 'ImpairmentOfCapitalAssets（资本资产减值）',
 'InterestExpense（利息费用）',
 'InterestExpenseNonOperating（非经营性利息费用）',
 'InterestIncome（利息收益）',
 'InterestIncomeNonOperating（非经营性利息收入）',
 'MinorityInterests（少数股东权益）',
 'NetIncome（净利润）',
 'NetIncomeCommonStockholders（归属于普通股股东的净利润，用于计算EPS和PE）',
 'NetIncomeContinuousOperations（扣非后净利润）',
 'NetIncomeFromContinuingAndDiscontinuedOperation（来自持续经营和停止经营业务的净收入）',
 'NetIncomeFromContinuingOperationNetMinorityInterest（不归属少数股东的扣非后净利润？）',
 'NetIncomeIncludingNoncontrollingInterests（包括非控股权的净收入？）',
 'NetInterestIncome（净利息收入）',
 'NetNonOperatingInterestIncomeExpense（？）',
 'NormalizedEBITDA（调整后EBITDA？）',
 'NormalizedIncome（调整后利润？）',
 'OperatingExpense（营业费用）',
 'OperatingIncome（营业利润）',
 'OperatingRevenue（营业收入）',
 'OtherNonOperatingIncomeExpenses（其他营业外收入支出？）',
 'OtherOperatingExpenses（其它营业费用）',
 'OtherSpecialCharges（其他特殊费用）',
 'OtherunderPreferredStockDividend（优先股股利下的其他项目）',
 'PretaxIncome（税前利润）',
 'ReconciledCostOfRevenue（对账后的经营收入成本？）',
 'ReconciledDepreciation（对账后的折旧）',
 'RentAndLandingFees（租金及土地费用？）',
 'RentExpenseSupplemental（补充租金费用？）',
 'ResearchAndDevelopment（研发费用？）',
 'SellingAndMarketingExpense（销售和市场营销费用）',
 'SellingGeneralAndAdministration（销售及一般管理费用）',
 'SpecialIncomeCharges（特殊的收入费用？）',
 'TaxEffectOfUnusualItems（特殊项目的税收效应？）',
 'TaxProvision（税金？）',
 'TaxRateForCalcs（计算用的税率）',
 'TotalExpenses（总费用）',
 'TotalOperatingIncomeAsReported（报告的总营业利润）',
 'TotalOtherFinanceCost（其他财务成本合计）',
 'TotalRevenue（总收入）',
 'TotalUnusualItems（非经常性项目总计）',
 'TotalUnusualItemsExcludingGoodwill（不包括商誉的非经常项目合计）',
 'WriteOff（冲销？）',
 
 'BeginningCashPosition（期初现金头寸）',
 'CapitalExpenditure（资本支出）',
 'CashDividendsPaid（现金股利支付）',
 'ChangeInCashSupplementalAsReported（现金补充变更报告？）',
 'ChangeInInventory（存货变化）',
 'ChangeInWorkingCapital（营运资本的变动额）',
 'DepreciationAndAmortization（折旧摊销）',
 'EndCashPosition（期末现金头寸）',
 'FreeCashFlow（自有现金流）',
 'InvestingCashFlow（投资现金流）',
 'NetOtherFinancingCharges（其他融资费用净额）',
 'NetOtherInvestingChanges（其他投资变动净额）',
 'OperatingCashFlow（营运现金流）',
 'OtherNonCashItems（其他非现金项目）'
 ]
"""

#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================



# -*- coding: utf-8 -*-

"""
版权：王德宏，北京外国语大学国际商学院
功能：计算CAPM模型贝塔系数的调整值
版本：2.1，2019-7-25
"""

#==============================================================================
#关闭所有警告
import warnings; warnings.filterwarnings('ignore')
from siat.common import *
from siat.translate import *
from siat.grafix import *
from siat.security_prices import *
#==============================================================================
import matplotlib.pyplot as plt

#处理绘图汉字乱码问题
import sys; czxt=sys.platform
if czxt in ['win32','win64']:
    plt.rcParams['font.sans-serif'] = ['SimHei']  # 设置默认字体
    mpfrc={'font.family': 'SimHei'}

if czxt in ['darwin']: #MacOSX
    plt.rcParams['font.family']= ['Heiti TC']
    mpfrc={'font.family': 'Heiti TC'}

if czxt in ['linux']: #website Jupyter
    plt.rcParams['font.family']= ['Heiti TC']
    mpfrc={'font.family':'Heiti TC'}

# 解决保存图像时'-'显示为方块的问题
plt.rcParams['axes.unicode_minus'] = False 
#==============================================================================
def prepare_capm(stkcd,mktidx,start,end):
    """
    函数功能：准备计算一只股票CAPM模型贝塔系数的数据，并标记年度
    输入参数：
    stkcd: 股票代码
    mktidx: 指数代码
    start：使用股票价格数据的开始日期，MM/DD/YYYY
    end：使用股票价格数据的结束日期，MM/DD/YYYY
    输出数据：
    返回数据：带年度标记的可直接用于capm回归的股票收益率数据
    """
        
    #仅用于调试，正式使用前应注释掉
    #stkcd='002504.SZ'; mktidx='000001.SS'
    #start="12/31/2011"; end="12/31/2018"

    #抓取股价和指数
    stock=get_price(stkcd,start,end)
    if stock is None:
        print("  #Error(prepare_capm): no data retrieved from server!")
        return None
    market=get_price(mktidx,start,end)
    if market is None:
        print("  #Error(prepare_capm): no index data retrieved from server!")
        return None    

    #计算日收益率
    import pandas as pd
    stkret=pd.DataFrame(stock['Close'].pct_change())
    mktret=pd.DataFrame(market['Close'].pct_change())

    #合并，去掉空缺
    R=pd.merge(mktret,stkret,how='left',left_index=True,right_index=True)
    R=R.dropna()

    #标记各个年度
    R['Year']=R.index.strftime("%Y")

    #返回带年份的股票收益率序列
    return R

if __name__=='__main__':
    R1=prepare_capm('0700.HK','^HSI','2014-01-01','2018-12-31')

#==============================================================================
#==============================================================================
def get_beta_ML(stkcd,mktidx,yearlist,printout=True,graph=True):
    """
    函数功能：使用ML方法调整一只股票的CAPM模型贝塔系数
    输入参数：
    stkcd: 股票代码
    mktidx: 指数代码
    yearlist：年度列表，列出其中期间的贝塔系数
    输出数据：
    显示CAPM市场模型回归的beta, 以及ML调整后的beta系数
    返回数据:年度CAPM贝塔系数和ML调整后的beta系数
    """

    #仅为测试用，完成后应立即注释掉
    #stkcd='0700.HK'
    #mktidx='^HSI'
    #yearlist=['2015','2016','2017','2018']
    
    Y4=str(int(yearlist[0])-1)
    start=Y4+'-01-01'
    end=yearlist[-1]+'-12-31'    
        
    #读取股价并准备好收益率数据
    try:
        R=prepare_capm(stkcd,mktidx,start,end)
    except:
        print("Error #1(get_beta_ML): Preparing CAPM data failed!")
        return None

    if (R is None):
        print("Error #2(get_beta_ML): server time out")
        print("Possible reasons:")  
        print("  1)internet connection problem")            
        print("  2)incorrect parameters for function")            
        print("  3)server too busy")            
        return None
    if (len(R) == 0):
        print("Error #3(get_beta_ML): server returned empty data")
        print("Possible reasons: data tentatively inaccessible for non-US stock")              
        return None
    
    #用于保存beta(CAPM)和beta(ML)
    import pandas as pd
    betas=pd.DataFrame(columns=('Year','Beta(CAPM)','Beta(ML)'))

    #计算Merrill-Lynch方法贝塔系数调整
    from scipy import stats
    for year in yearlist:
        r=R[R['Year']==year]
        if len(r) != 0:
            output=stats.linregress(r['Close_x'],r['Close_y'])
            (beta,alpha,r_value,p_value,std_err)=output
            beta_ML=beta*2.0/3.0+1.0/3.0
            #整齐输出 
            #print(year,"%6.4f "%(beta),"%6.4f "%(beta_ML))

            row=pd.Series({'Year':year,'Beta(CAPM)':beta,'Beta(ML)':beta_ML})
            betas=betas.append(row,ignore_index=True)

    betas.set_index(["Year"], inplace=True)
    
    if printout == True: printdf(betas,2)
    if graph == True:
        model="贝塔系数的简单调整法"
        draw2_betas(model,mktidx,stkcd,betas)

    return betas

#==============================================================================
def printdf(df,decimal=2):
    """
    功能：整齐地显示数据框的内容，自动调整各列宽度
    """
    #打印时保留的小数点位数
    dec="%."+str(decimal)+"f"
    format=lambda x: dec % x
    df1=df.applymap(format)    
    
    import pandas as pd
    #调整最佳列宽
    old_width = pd.get_option('display.max_colwidth')
    pd.set_option('display.max_colwidth', -1)
    print(df1)
    pd.set_option('display.max_colwidth', old_width)

    return
    
if __name__=='__main__':
    yearlist=gen_yearlist['2010','2019']
    betas=get_beta_ML('AAPL','^GSPC',yearlist)    
    betas2=get_beta_ML('BILI','^GSPC',yearlist)
    betas3=get_beta_ML('0700.HK','^HSI',yearlist)
    yearlist1=['2015','2016','2017','2018']
    betas3=get_beta_ML('0700.HK','^HSI',yearlist1)

#==============================================================================
def draw2_betas(model,scope,ticker,betas):    
    """
    功能：绘制双曲线的贝塔因子变化图
    输入参数：
    model: 模型类型, 任意字符串(例如Merrill-Lynch Beta Adjustment)
    scope: 市场指数, 任意字符串(例如Standard & Poor 500)
    ticker：股票代码
    输出：图形
    """
    #仅用作测试，完成后应注释掉
    #model="Merrill-Lynch Beta Adjustment"
    #scope="Standard & Poor 500"
    #ticker="AAPL"

    #取得股票和指数名字，对于非美股可能耗时较长
    """
    import yfinance as yf
    mktidx= yf.Ticker(scope)
    idxinfo=mktidx.info
    idxname=idxinfo['shortName']
    stkcd=yf.Ticker(ticker)
    stkinfo=stkcd.info
    stkname=stkinfo['shortName']   
    title1="\n"+stkname+"\n"+model+"\n(Benchmark on "+idxname+")"
    """
    title1=codetranslate(ticker)+": "+model+"\n(基于"+codetranslate(scope)+")"
   
    #转换索引类型为DatetimeIndex，便于后续处理
    """
    import pandas as pd
    betas['Date']=betas.index
    betas['Date']=pd.to_datetime(betas['Date'])
    betas.set_index('Date',inplace=True)
    """

    #获得列明
    betalist=betas.columns.values.tolist()
    beta1=betalist[0]
    beta2=betalist[1]

    try:
        plt.plot(betas[beta1],label=beta1,marker='o',color='red')
        plt.plot(betas[beta2],label=beta2,marker='*',linewidth=2,ls='-.',color='blue')
    except:
        print("Error #1(draw2_betas) no available data for drawing!")
        return
    plt.axhline(y=1.0,color='b',linestyle=':',label='市场线')  
    plt.title(title1,fontsize=12,fontweight='bold')
    plt.ylabel("贝塔系数",fontsize=12,fontweight='bold')
    plt.xticks(rotation=45)
    plt.legend(loc='best')    
    
    import datetime; today = datetime.date.today()
    plt.xlabel("数据来源：雅虎财经，"+str(today))    
    
    plt.show()       
    
    return

if __name__=='__main__':
    model="ML Beta Adjustment"
    scope="SP500"
    ticker="AAPL"
    draw2_betas(model,scope,ticker,betas)


#==============================================================================
def get_beta_SW(stkcd,mktidx,yearlist,printout=True,graph=True):
    """
    函数功能：使用SW方法调整一只股票的CAPM模型贝塔系数
    输入参数：
    stkcd: 股票代码
    mktidx: 指数代码
    yearlist：年度列表，列出其中期间的贝塔系数
    输出数据：显示CAPM市场模型回归的beta, 以及调整后的beta系数
    返回数据：CAPM市场模型回归的beta, 以及调整后的beta系数
    """

    #仅为测试用，完成后应立即注释掉
    #stkcd='0700.HK'
    #mktidx='^HSI'
    #yearlist=['2015','2016','2017','2018']
    
    #生成开始结束日期
    Y4=str(int(yearlist[0])-1)
    start=Y4+'-01-01'
    end=yearlist[-1]+'-12-31'   
        
    #读取股价并准备好收益率数据
    try:
        R=prepare_capm(stkcd,mktidx,start,end)
    except:
        print("Error #1(get_beta_SW): preparing CAPM data failed!")
        return None

    if (R is None):
        print("Error #2(get_beta_SW): server time out")
        print("Possible reasons:")  
        print("  1)internet connection problem")            
        print("  2)incorrect parameters for function")            
        print("  3)server too busy")            
        return None
    if (len(R) == 0):
        print("Error #3(get_beta_SW): server returned empty data")
        print("Possible reasons: data tentatively inaccessible for non-US stock")              
        return None

    #用于保存beta(CAPM)和beta(SW)
    import pandas as pd
    betas=pd.DataFrame(columns=('Year','Beta(CAPM)','Beta(SW)'))

    #计算Scholes-William调整
    R['Close_x+1']=R['Close_x'].shift(1)    
    R['Close_x-1']=R['Close_x'].shift(-1)
    R=R.dropna()    #stats.linregress不接受空缺值

    from scipy import stats    
    for year in yearlist:
        r=R[R['Year']==year]
        if len(r) != 0:
            output=stats.linregress(r['Close_x'],r['Close_y'])
            (beta0,alpha,r_value,p_value,std_err)=output

            output=stats.linregress(r['Close_x+1'],r['Close_y'])
            (beta1,alpha,r_value,p_value,std_err)=output 

            output=stats.linregress(r['Close_x-1'],r['Close_y'])
            (beta_1,alpha,r_value,p_value,std_err)=output    

            output=stats.linregress(r['Close_x-1'],r['Close_x'])
            (rou,alpha,r_value,p_value,std_err)=output    

            beta_SW=(beta_1+beta0+beta1)/(1.0+2.0*rou)
            row=pd.Series({'Year':year,'Beta(CAPM)':beta0,'Beta(SW)':beta_SW})
            betas=betas.append(row,ignore_index=True)            
    
    betas.set_index(["Year"], inplace=True)
    
    if printout == True: printdf(betas,2)
    if graph == True:
        model="贝塔系数的Scholes-Williams调整法"
        draw2_betas(model,mktidx,stkcd,betas)
    
    return betas

    
if __name__=='__main__':
    yearlist=gen_yearlist('2010','2019')
    betas_AAPL=get_beta_SW('AAPL','^GSPC',yearlist)
    
    model="SW Beta Adjustment"
    scope="SP500"
    ticker="AAPL"
    draw2_betas(model,scope,ticker,betas_AAPL)

#==============================================================================
def get_beta_dimson(stkcd,mktidx,yearlist,printout=True,graph=True):
    """
    函数功能：使用Dimson(1979)方法调整一只股票的CAPM模型贝塔系数
    输入参数：
    stkcd: 股票代码
    mktidx: 指数代码
    yearlist：年度列表，用于计算年度贝塔系数
    输出数据：显示CAPM市场模型回归的beta, 以及调整后的beta系数
    返回数据：CAPM的beta, 以及调整后的beta系数
    """

    #仅为测试用，完成后应立即注释掉
    #stkcd='0700.HK'
    #mktidx='^HSI'
    #yearlist=['2015','2016','2017','2018']
    
    #生成开始结束日期
    Y4=str(int(yearlist[0])-1)
    start=Y4+'-01-01'
    end=yearlist[-1]+'-12-31'  
        
    #读取股价并准备好收益率数据
    try:
        R=prepare_capm(stkcd,mktidx,start,end)
    except:
        print("  #Error(get_beta_dimson): preparing CAPM data failed!")
        return None

    if (R is None):
        print("  #Error(get_beta_dimson): server did not respond")
        """
        print("Possible reasons:")  
        print("  1)internet connection problem")            
        print("  2)incorrect parameters for function")            
        print("  3)server too busy")     
        """
        return None
    if (len(R) == 0):
        print("  #Error(get_beta_dimson): server returned empty data")
        print("  Possible reasons: data tentatively inaccessible for non-US stock")              
        return None

    #用于保存beta(CAPM)和beta(Dimson)
    import pandas as pd
    betas=pd.DataFrame(columns=('Year','Beta(CAPM)','Beta(Dimson)'))

    #计算Dimson(1979)调整
    R['Close_x+1']=R['Close_x'].shift(1)    
    R['Close_x-1']=R['Close_x'].shift(-1)   
    R=R.dropna()

    from scipy import stats    
    import statsmodels.api as sm
    for year in yearlist:
        r=R[R['Year']==year]
        if len(r) != 0:   
            output=stats.linregress(r['Close_x'],r['Close_y'])
            (beta_capm,alpha,r_value,p_value,std_err)=output               
            
            #三个解释变量
            RX=r[['Close_x-1','Close_x','Close_x+1']]
            X1=sm.add_constant(RX)	#要求回归具有截距项
            Y=r['Close_y']
            model = sm.OLS(Y,X1)	#定义回归模型，X1为多元矩阵
            results = model.fit()	#进行OLS回归

            (alpha,beta_1,beta0,beta1)=results.params	#提取回归系数
            beta_dimson=beta_1+beta0+beta1            

            row=pd.Series({'Year':year,'Beta(CAPM)':beta_capm, \
                           'Beta(Dimson)':beta_dimson})
            betas=betas.append(row,ignore_index=True)               

    betas.set_index(["Year"], inplace=True)

    if printout == True: printdf(betas,2)
    if graph == True:
        model="贝塔系数的Dimson调整法"
        draw2_betas(model,mktidx,stkcd,betas)

    return betas
    
if __name__=='__main__':
    yearlist=gen_yearlist('2010','2019')
    betas_MSFT=get_beta_dimson('MSFT','^GSPC',yearlist)
    
    model="Dimson Beta Adjustment"
    scope="SP500"
    ticker="MSFT"
    draw2_betas(model,scope,ticker,betas_MSFT)

    betas_MSFT2=get_beta_dimson('MSFT','^DJI',yearlist)
    
    model="Dimson Beta Adjustment"
    scope="DJIA"
    ticker="MSFT"
    draw2_betas(model,scope,ticker,betas_MSFT2)

#==============================================================================
#==============================================================================
#==============================================================================

def prepare_hamada_patch_is(ticker):
    """
    在雅虎财经接口获取利润表数据失败时，改从tushare获取
    获取的项目：所得税费用，税前利润
    """
    import pandas as pd
    import tushare as ts
    pro=init_ts()
    
    #财报期限
    import datetime
    today=datetime.date.today()
    thisyear=today.year
    fouryrsago=thisyear-4-2
    fromdate=str(fouryrsago)+"0101"
    todate=str(thisyear)+"1231"

    #利润表
    ticker1=ticker.upper()
    suffix=ticker1[-3:]
    if suffix in ['.SS']:
        ticker1=ticker1.replace('.SS','.SH',1)
    if not (suffix in ['.SS','.SZ']):
        print("#Erro(prepare_hamada_patch_is): no financials available for",ticker)
        return None
    
    fis=pro.income(ts_code=ticker1,start_date=fromdate,end_date=todate)
    #去除非年报
    stripfmt=lambda x:(x.strip())[-4:]
    fis['yrtag']=fis['end_date'].apply(stripfmt)
    fis1=fis[fis['report_type']=='1']   #合并报表
    fis2=fis1[fis1['yrtag']=="1231"]    #年报
    fis3=fis2.drop_duplicates(subset=['end_date'],keep='first') #去重
    fis4=fis3   #保留最新4年，与yfinance结果保持一致
    fis4.sort_values(by=['end_date'],ascending=True,inplace=True)   #升序排序

    #重建索引
    fis4['date']=pd.to_datetime(fis4['end_date'])
    fis4.set_index('date',inplace=True)

    #提取需要的项目：所得税费用，税前利润
    fis4['Income Tax Expense']=fis4['income_tax']
    fis4['Income Before Tax']=fis4['total_profit']
    fis5=fis4[['Income Tax Expense','Income Before Tax']].copy()

    return fis5

if __name__=='__main__':
    ticker="600519.SS"    


#==============================================================================
def prepare_hamada_patch_bs(ticker):
    """
    在雅虎财经接口获取资产负债表数据失败时，改从tushare获取
    获取的项目：负债合计，股东权益合计
    """
    import pandas as pd
    import tushare as ts
    pro=init_ts()
    
    #财报期限
    import datetime
    today=datetime.date.today()
    thisyear=today.year
    fouryrsago=thisyear-4-2
    fromdate=str(fouryrsago)+"0101"
    todate=str(thisyear)+"1231"

    #利润表
    ticker1=ticker.upper()
    suffix=ticker1[-3:]
    if suffix in ['.SS']:
        ticker1=ticker1.replace('.SS','.SH',1)
    if not (suffix in ['.SS','.SZ']):
        print("#Error(prepare_hamada_patch_bs): no financials available for",ticker)
        return None
    
    fis=pro.balancesheet(ts_code=ticker1,start_date=fromdate,end_date=todate)
    #去除非年报
    stripfmt=lambda x:(x.strip())[-4:]
    fis['yrtag']=fis['end_date'].apply(stripfmt)
    fis1=fis[fis['report_type']=='1']   #合并报表
    fis2=fis1[fis1['yrtag']=="1231"]    #年报
    fis3=fis2.drop_duplicates(subset=['end_date'],keep='first') #去重
    fis4=fis3   #保留最新4年，与yfinance结果保持一致
    fis4.sort_values(by=['end_date'],ascending=True,inplace=True)   #升序排序

    #重建索引
    fis4['date']=pd.to_datetime(fis4['end_date'])
    fis4.set_index('date',inplace=True)

    #提取需要的项目：所得税费用，税前利润
    fis4['Total Liab']=fis4['total_liab']
    fis4['Total Stockholder Equity']=fis4['total_hldr_eqy_inc_min_int']
    fis5=fis4[['Total Liab','Total Stockholder Equity']].copy()

    return fis5

if __name__=='__main__':
    ticker="600519.SS"    


#==============================================================================
if __name__ =="__main__":
    ticker='0700.HK'

def prepare_hamada_yearly_yahoo(ticker):
    """
    功能：从雅虎财经下载财报数据，计算hamada模型需要的因子
    局限：只能下载最近4年的财报
    输入：股票代码
    输出：
        寻找数据项：所得税费用，税前利润，计算实际税率；
        总负债，所有者权益，计算财务杠杆
    数据框, CFLB，贝塔Lev对贝塔Unlev的倍数
    年度列表
    """
    print("... Searching for financial information, please wait ...")
    import yfinance as yf
    stock=yf.Ticker(ticker)    

    #利润表
    try:
        is0=stock.financials
        is1=is0.T
    except:
        is1=prepare_hamada_patch_is(ticker)
    
    if len(is0)==0: #yfinance失效
        is1=prepare_hamada_patch_is(ticker)

    is1['income tax expense']=is1['Income Tax Expense'].astype('float')
    is1['income before tax']=is1['Income Before Tax'].astype('float')
    is1['tax rate']=is1['income tax expense']/is1['income before tax']

    import pandas as pd
    is1['date']=pd.to_datetime(is1.index)
    is1.set_index(["date"], inplace=True)
    is2=is1.sort_index(axis=0,ascending=True)
    tax=pd.DataFrame(is2['tax rate'])

    #资产负债表
    try:
        bs0=stock.balance_sheet
        bs1=bs0.T
    except:
        bs1=prepare_hamada_patch_bs(ticker)
    if len(bs0)==0: #yfinance失效
        bs1=prepare_hamada_patch_bs(ticker)    

    bs1['total liabilities']=bs1['Total Liab'].astype('float')
    bs1['total equities']=bs1["Total Stockholder Equity"].astype('float')
    bs1['lev ratio']=bs1['total liabilities']/bs1['total equities']
    bs1['date']=pd.to_datetime(bs1.index)
    bs1.set_index(['date'],inplace=True)
    bs2=bs1.sort_index(axis=0,ascending=True)
    lev=pd.DataFrame(bs2['lev ratio'])
    
    #合成，计算
    fac=pd.merge(lev,tax,how='left',left_index=True,right_index=True)
    fac['CFLB%']=1/(1+(1/fac['lev ratio'])*(1/abs(1-fac['tax rate'])))*100
    fac['lev_unlev']=1+fac['lev ratio']*(1-fac['tax rate'])
    fac['year']=fac.index.strftime("%Y")
    yearlist=list(fac['year'])

    return fac,yearlist

if __name__ =="__main__":
    ticker="600519.SS"
    ticker="AAPL"
    ticker='000002.SZ'
    fac,yl=prepare_hamada_yearly_yahoo("MSFT")

#==============================================================================
if __name__ =="__main__":
    stkcd='0700.HK'
    mktidx='^HSI'
    yearlist=['2015','2016','2017','2018']


def get_beta_hamada(stkcd,mktidx,yearlist,printout=True,graph=True):
    """
    函数功能：使用Hamada(1972)方法，计算无杠杆贝塔系数
    输入参数：
    stkcd: 股票代码
    mktidx: 指数代码
    yearlist：年度列表，用于计算年度贝塔系数
    输出数据：显示CAPM市场模型回归的beta, 以及调整后的beta系数
    返回数据：CAPM的beta, Hamada beta，CFLB(债务融资对CAPM beta系数的贡献率)
    """
    
    #计算Hamada参数，并返回可用的年度列表
    fac,yearlist=prepare_hamada_yearly_yahoo(stkcd)
    if fac is None:
        print("#Error(get_beta_hamada): no financial info available for",stkcd)
        return None
    
    #生成开始结束日期
    Y4=str(int(yearlist[0])-1)
    start=Y4+'-01-01'
    end=yearlist[-1]+'-12-31'  
        
    #读取股价并准备好收益率数据
    try:
        R=prepare_capm(stkcd,mktidx,start,end)
    except:
        print("#Error(get_beta_hamada): preparing CAPM data failed!")
        print("Info:",stkcd,mktidx,yearlist)              
        return None

    if (R is None):
        print("#Error(get_beta_hamada): server time out")
        return None
    if (len(R) == 0):
        print("#Error(get_beta_hamada): server returned empty data")
        print("Perhaps data tentatively inaccessible for non-US stock")              
        return None
    R=R.dropna()
    
    #用于保存beta(CAPM)和beta(Hamada)
    import pandas as pd
    betas=pd.DataFrame(columns=('Year','Beta(CAPM)','Beta(Unlevered)','CFLB%'))

    from scipy import stats    
    for year in yearlist:
        r=R[R['Year']==year]
        if len(r) != 0:   
            output=stats.linregress(r['Close_x'],r['Close_y'])
            (beta_capm,alpha,r_value,p_value,std_err)=output               
            
            #Hamada无杠杆因子
            lev_unlev=fac[fac['year']==year]['lev_unlev'].values[0]
            beta_hamada=beta_capm/lev_unlev
            cflb=fac[fac['year']==year]['CFLB%'].values[0]            

            row=pd.Series({'Year':year,'Beta(CAPM)':beta_capm, \
                           'Beta(Unlevered)':beta_hamada,'CFLB%':cflb})
            betas=betas.append(row,ignore_index=True)               

    betas.set_index(["Year"], inplace=True)

    if printout == True: 
        printdf(betas,2)
    if graph == True:
        model="Hamada Unlevered Beta"
        draw2_betas(model,mktidx,stkcd,betas)
        
        #绘制CFLB
        if len(betas)<=1: return betas
        
        plt.plot(betas['CFLB%'],marker='o',color='red',lw=3)
        
        bmin=min(list(betas['CFLB%']))
        bmax=max(list(betas['CFLB%']))
        axhmin=(int(bmin/10)+1)*10
        if bmin <= axhmin <= bmax:
            plt.axhline(y=axhmin,color='b',linestyle=':') 
        axhmax=(int(bmax/10))*10
        if bmin <= axhmax <= bmax:
            plt.axhline(y=axhmax,color='b',linestyle=':')         
        
        title1=stkcd+": Contribution of Financial Leverage to Beta"+ \
            "\n(Benchmark on Market Index "+mktidx+")"
        plt.title(title1,fontsize=12,fontweight='bold')
        plt.ylabel("CFLB %",fontsize=12,fontweight='bold')
        #plt.legend(loc='best')         
        plt.grid(ls='-.')
        #查看可用的样式：print(plt.style.available)
        #样式：bmh(好),classic,ggplot(好，图大)，tableau-colorblind10，
        #样式：seaborn-bright，seaborn-poster，seaborn-whitegrid
        plt.style.use('bmh')
        plt.show()         

    return betas
    
if __name__=='__main__':
    stkcd='000002.SZ'
    mktidx='000001.SS'
    yearlist=gen_yearlist('2010','2019')
    betas1=get_beta_hamada('MSFT','^GSPC',yearlist)

#==============================================================================
#==============================================================================
#==============================================================================

def prepare_hamada_yearly_ts(ticker):
    """
    功能：从tushare下载财报数据，计算hamada模型需要的因子
    局限：
    输入：股票代码
    输出：
    数据框, CFLB，贝塔Lev对贝塔Unlev的倍数
    年度列表
    """
    #仅限测试使用    
    #ticker='600519.SS'
    
    #转换股票代码.SS为.SH(tushare使用.SH而不是雅虎的.SS)
    ticker1=ticker.upper()
    try: ticker2=ticker1.replace('.SS','.SH')
    except: pass

    #初始化tushare
    pro=init_ts() 
    
    #利润表
    is0=pro.income(ts_code=ticker2)
    
    #修改报表截止日期格式为YYYY-MM-DD
    is0['year']=is0.apply(lambda x:x['end_date'][0:4],axis=1)
    is0['MM']=is0.apply(lambda x:x['end_date'][4:6],axis=1)
    is0['DD']=is0.apply(lambda x:x['end_date'][6:8],axis=1)
    is0['date']=is0['year']+'-'+is0['MM']+'-'+is0['DD']
    
    #只取年报              
    is1=is0[is0['MM']=='12']
    #最新的合并报表
    is1=is1[is1['report_type']=='1'] 
    #去掉重复值
    is1.drop_duplicates(subset=['ts_code','end_date','report_type'], \
                        keep='first',inplace=True)
    #计算实际所得税率
    is1['income tax expense']=is1['income_tax'].astype('float')
    is1['income before tax']=is1['total_profit'].astype('float')
    is1['tax rate']=is1['income tax expense']/is1['income before tax']

    is1.set_index(["year"], inplace=True)
    is2=is1.sort_index(axis=0,ascending=True)
    import pandas as pd
    tax=pd.DataFrame(is2['tax rate'])

    #资产负债表
    bs0=pro.balancesheet(ts_code=ticker2)
    
    #修改报表截止日期格式为YYYY-MM-DD
    bs0['year']=bs0.apply(lambda x:x['end_date'][0:4],axis=1)
    bs0['MM']=bs0.apply(lambda x:x['end_date'][4:6],axis=1)
    bs0['DD']=bs0.apply(lambda x:x['end_date'][6:8],axis=1)
    bs0['date']=bs0['year']+'-'+bs0['MM']+'-'+bs0['DD']    
    
    #只取年报              
    bs1=bs0[bs0['MM']=='12']
    #最新的合并报表
    bs1=bs1[bs1['report_type']=='1'] 
    #去掉重复值
    bs1.drop_duplicates(subset=['ts_code','end_date','report_type'], \
                        keep='first',inplace=True)
    
    bs1['total liabilities']=bs1['total_liab'].astype('float')
    bs1['total equities']=(bs1["total_assets"]-bs1["total_liab"]).astype('float')
    bs1['lev ratio']=bs1['total liabilities']/bs1['total equities']
    bs1.set_index(['year'],inplace=True)
    bs2=bs1.sort_index(axis=0,ascending=True)
    lev=pd.DataFrame(bs2['lev ratio'])
    
    #合成，计算
    fac=pd.merge(lev,tax,how='left',left_index=True,right_index=True)
    fac['CFLB%']=1/(1+(1/fac['lev ratio'])*(1/abs(1-fac['tax rate'])))*100
    fac['lev_unlev']=1+fac['lev ratio']*(1-fac['tax rate'])
    fac['year']=fac.index
    yearlist=list(fac['year'])

    return fac,yearlist

if __name__ =="__main__":
    fac,yl=prepare_hamada_yearly_ts("MSFT")

#==============================================================================
def get_beta_hamada_ts(stkcd,mktidx,yearlist,printout=True,graph=True):
    """
    函数功能：基于Hamada(1972)方法，使用tushare数据，计算无杠杆贝塔系数
    输入参数：
    stkcd: 股票代码
    mktidx: 指数代码
    yearlist：年度列表，用于计算年度贝塔系数
    输出数据：显示CAPM市场模型回归的beta, 以及调整后的beta系数
    返回数据：CAPM的beta, Hamada beta，CFLB(债务融资对CAPM beta系数的贡献率)
    """

    #仅为测试用，完成后应立即注释掉
    """
    stkcd='600519.SS'
    mktidx='000001.SS'
    yearlist=gen_yearlist('2011','2019')
    """
    
    #计算Hamada参数，并返回可用的年度列表
    fac,yearlist2=prepare_hamada_yearly_ts(stkcd)
    fac=fac[fac['year']>=yearlist[0]]
    fac=fac[fac['year']<=yearlist[-1]]
    
    #生成开始结束日期
    Y4=str(int(yearlist[0])-1)
    start=Y4+'-01-01'
    end=yearlist[-1]+'-12-31'  
        
    #读取股价并准备好收益率数据
    try:
        R=prepare_capm(stkcd,mktidx,start,end)
    except:
        print("Error #1(get_beta_hamada_ts): preparing CAPM data failed!")
        print("Information:",stkcd,mktidx,yearlist)              
        return None

    if (R is None):
        print("Error #2(get_beta_hamada_ts): server time out")
        print("Possible reasons:")  
        print("  1)internet connection problem")            
        print("  2)incorrect parameters for function")            
        print("  3)server too busy")            
        return None
    if (len(R) == 0):
        print("Error #3(get_beta_hamada_ts): server returned empty data")
        print("Possible reasons: data tentatively inaccessible for non-US stock")              
        return None
    R=R.dropna()
    
    #用于保存beta(CAPM)和beta(Hamada)
    import pandas as pd
    betas=pd.DataFrame(columns=('Year','Beta(CAPM)','Beta(Unlevered)', \
                                'CFLB%','Tax Rate%','Debt Ratio%'))

    from scipy import stats    
    for year in yearlist:
        r=R[R['Year']==year]
        if len(r) == 0: continue  
        output=stats.linregress(r['Close_x'],r['Close_y'])
        (beta_capm,alpha,r_value,p_value,std_err)=output               
            
        #Hamada无杠杆因子
        try:
            lev_unlev=fac[fac['year']==year]['lev_unlev'].values[0]
        except: continue
        beta_hamada=beta_capm/lev_unlev
        cflb=fac[fac['year']==year]['CFLB%'].values[0] 
        taxrate=fac[fac['year']==year]['tax rate'].values[0]*100 
        levratio=fac[fac['year']==year]['lev ratio'].values[0]*100

        row=pd.Series({'Year':year,'Beta(CAPM)':beta_capm, \
                           'Beta(Unlevered)':beta_hamada,'CFLB%':cflb, \
                           'Tax Rate%':taxrate,'Debt Ratio%':levratio})
        betas=betas.append(row,ignore_index=True)               

    betas.set_index(["Year"], inplace=True)

    if printout == True: 
        printdf(betas,2)
    if graph == True:
        model="Hamada Unlevered Beta"
        draw2_betas(model,mktidx,stkcd,betas)
        
        #绘制Hamada影响因子
        draw_hamada_factors(stkcd,mktidx,betas)        

    return betas
    
if __name__=='__main__':
    yearlist=gen_yearlist('2010','2019')
    betas1=get_beta_hamada('MSFT','^GSPC',yearlist)
#==============================================================================
def draw_hamada_factors(stkcd,mktidx,betas):
    """
    功能：绘制Hamada模型因子的变化折线图，企业实际所得税税率，资产负债率，CFLB
    """
    if len(betas)<=1: return
    
    #计算资产负债率：由 D/E到 D/(A=D+E)
    betas['Debt/Assets%']=1/(1+1/(betas['Debt Ratio%']/100))*100

    #fig=plt.figure(figsize=(8,6))
    fig=plt.figure()
    ax1=fig.add_subplot(111)
    ax1.plot(betas['CFLB%'],marker='o',color='green',lw=3,label='CFLB%')
    ax1.plot(betas['Debt/Assets%'],marker='o',color='red',lw=2,ls='--', \
             label='Debt/Assets%')
    ax1.set_ylabel("CFLB%, Debt/Assets%")
    ax1.legend(loc='upper left') 
    ax1.set_xticklabels(betas.index,rotation=45)
    
    ax2=ax1.twinx()
    ax2.plot(betas['Tax Rate%'],marker='o',color='black',lw=2,ls='-.', \
             label='Income Tax%')
    ax2.set_ylabel('Income Tax%')  
    ax2.legend(loc='lower right')
    ax2.set_xticklabels(betas.index,rotation=45)
    
    title1=stkcd+": Impact of Hamada Factors on Beta"+ \
            "\n(Benchmark on Market Index "+mktidx+")"
    plt.title(title1,fontsize=12,fontweight='bold')
    plt.style.use('ggplot')
    plt.show()     
    
    return
#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
if __name__ =="__main__":
    stkcd='0700.HK'
    stkcd='AAPL'
    stkcd='GS'
    stkcd='BA'
    mktidx='^HSI'
    stkcd='000002.SZ'
    stkcd='600606.SS'
    mktidx='000001.SS'

def get_beta_hamada2(stkcd,mktidx,printout=True,graph=True):
    """
    函数功能：使用Hamada(1972)方法，计算无杠杆贝塔系数，绘图
    输入参数：
    stkcd: 股票代码
    mktidx: 指数代码
    输出数据：显示CAPM市场模型回归的beta, 以及调整后的beta系数
    返回数据：CAPM的beta, Hamada beta，CFLB(债务融资对CAPM beta系数的贡献率)
    """
    
    #计算Hamada参数，并返回可用的年度列表
    fac=prepare_hamada_yahoo(stkcd)
    if fac is None:
        print("  #Error(get_beta_hamada2): no financial info available for",stkcd)
        return None
    datecvt=lambda x: str(x.strftime("%Y-%m-%d"))
    fac['fsdate']=fac.index.date
    fac['fsdate']=fac['fsdate'].apply(datecvt)
    
    #生成开始结束日期
    end0=fac.index[-1]
    end=date_adjust(end0,adjust=14)
    start0=fac.index[0]
    start=date_adjust(start0,adjust=-366)
        
    #读取股价并准备好收益率数据
    try:
        R=prepare_capm(stkcd,mktidx,start,end)
    except:
        print("  #Error(get_beta_hamada2): preparing CAPM data failed for",stkcd,mktidx)
        return None
    if (R is None) or (len(R) == 0):
        print("  #Error(get_beta_hamada2): retrieved empty info in CAPM for",stkcd,mktidx)
        return None
    R=R.dropna()
    R['prcdate']=R.index.date
    R['prcdate']=R['prcdate'].apply(datecvt)
    
    #用于保存beta(CAPM)和beta(Hamada)
    import pandas as pd
    betas=pd.DataFrame(columns=('Date','Beta(CAPM)','Beta(Unlevered)','CFLB%'))
    fsdatelist=list(fac['fsdate'])
    from scipy import stats    
    for d in fsdatelist:
        dstart=date_adjust(d,adjust=-365)
        r=R[R['prcdate'] >= dstart]
        r=r[r['prcdate'] <= d]
        if len(r) != 0:   
            output=stats.linregress(r['Close_x'],r['Close_y'])
            (beta_capm,alpha,r_value,p_value,std_err)=output               
            
            #Hamada无杠杆因子
            lev_unlev=fac[fac['fsdate']==d]['lev_unlev'].values[0]
            beta_hamada=beta_capm/lev_unlev
            cflb=fac[fac['fsdate']==d]['CFLB%'].values[0]            

            row=pd.Series({'Date':d,'Beta(CAPM)':beta_capm, \
                           'Beta(Unlevered)':beta_hamada,'CFLB%':cflb})
            betas=betas.append(row,ignore_index=True)               
    betas.set_index(["Date"], inplace=True)

    #打印
    if printout == True: 
        printdf(betas,2)
    
    #绘图：两种杠杆对比图，CFLB图
    if graph == True:
        
        #绘制Hamada对比图
        model="滨田无杠杆贝塔系数"
        draw2_betas(model,mktidx,stkcd,betas)
        
        #绘制CFLB单图
        if len(betas)<=1: 
            print("  #Notice(get_beta_hamada2): too few info for graphics of",stkcd)
            return betas
        
        plt.plot(betas['CFLB%'],marker='o',color='red',lw=3,label='CFLB%')
        
        #绘制均值虚线
        cflb_avg=betas['CFLB%'].mean()
        cflb_avg_txt='均值: '+str(round(cflb_avg,1))+'%'
        plt.axhline(y=cflb_avg,color='b',linestyle=':',label=cflb_avg_txt)
        
        title1=codetranslate(stkcd)+": 财务杠杆对于贝塔系数的贡献度(CFLB)"
        #plt.title(title1,fontsize=12,fontweight='bold')
        plt.title(title1)
        #plt.ylabel("CFLB %",fontsize=12,fontweight='bold')
        footnote="注: 基于"+codetranslate(mktidx)
        
        import datetime; today = datetime.date.today()
        footnote2="\n数据来源: 雅虎财经,"+str(today)
        plt.xlabel(footnote+footnote2)
        
        plt.grid(ls='-.')
        #查看可用的样式：print(plt.style.available)
        #样式：bmh(好),classic,ggplot(好，图大)，tableau-colorblind10，
        #样式：seaborn-bright，seaborn-poster，seaborn-whitegrid
        plt.style.use('bmh')
        plt.gcf().autofmt_xdate() # 优化标注（自动倾斜）
        plt.legend(loc='best')
        plt.show(); plt.close()
        
        #绘制CFLB+财务杠杆双图
        df1=betas; df2=fac.set_index(["fsdate"])
        ticker1=ticker2=stkcd
        colname1='CFLB%'; colname2='lev ratio'
        label1='CFLB%'; label2='财务杠杆'
        titletxt=codetranslate(stkcd)+": CFLB与财务杠杆之间的关系"
        footnote='注: 这里的财务杠杆使用的是负债/所有者权益'

        
        plot_line2_twinx(df1,ticker1,colname1,label1,df2,ticker2,colname2,label2, \
        titletxt,footnote+footnote2)
        
        #绘制CFLB+税率双图
        #df1=betas; df2=fac.set_index(["fsdate"])
        #ticker1=ticker2=stkcd
        colname1='CFLB%'; colname2='tax rate'
        label1='CFLB%'; label2='实际税率'
        titletxt=codetranslate(stkcd)+": CFLB与税率之间的关系"
        footnote='注: 这里使用的是实际税率'

        
        plot_line2_twinx(df1,ticker1,colname1,label1,df2,ticker2,colname2,label2, \
        titletxt,footnote+footnote2)
            
    return betas
    
if __name__=='__main__':
    betas1=get_beta_hamada2('MSFT','^GSPC')

#==============================================================================
if __name__ =="__main__":
    ticker='0700.HK'
    ticker="600519.SS"
    ticker="AAPL"
    ticker="BA"

def prepare_hamada_yahoo(ticker):
    """
    功能：从雅虎财经下载财报数据，计算hamada模型需要的因子
    局限：只能下载最近4年+4个季度的财报
    输入：股票代码
    输出：
        寻找数据项：所得税费用，税前利润，计算实际税率；
        总负债，所有者权益，计算财务杠杆
    数据框, CFLB，贝塔Lev对贝塔Unlev的倍数
    年度列表
    """
    print("...Searching for financial information, please wait ...")
    
    #利润表
    try:
        import siat.financial_statements as fs
        is1=fs.get_income_statements(ticker) 
    except:
        print("  #Error(prepare_hamada_yahoo): failed to retrieve income info of",ticker)
        return None
    if (is1 is None) or (len(is1)==0): 
        print("  #Error(prepare_hamada_yahoo): retrieve empty income info of",ticker)
        return None
    
    is1['tax rate']=is1['TaxRateForCalcs'].astype('float')
    is1['ticker']=is1.index
    is1['date']=is1['asOfDate']
    is1.set_index(["date"], inplace=True)
    is1.sort_index(axis=0,ascending=True,inplace=True)
    
    import pandas as pd
    tax=pd.DataFrame(is1['tax rate'])

    #资产负债表
    try:
        bs1=fs.get_balance_sheet(ticker)
    except:
        print("  #Error(prepare_hamada_yahoo): failed to retrieve balance sheet of",ticker)
        return None
    if (bs1 is None) or (len(bs1)==0):
        print("  #Error(prepare_hamada_yahoo): retrieve empty balance sheet of",ticker)
        return None

    bs1['lev ratio']=bs1['TotalLiabilities']/bs1['TotalEquities']
    bs1['date']=bs1['asOfDate']
    bs1.set_index(['date'],inplace=True)
    bs1.sort_index(axis=0,ascending=True,inplace=True)
    lev=pd.DataFrame(bs1['lev ratio'])
    
    #合成，计算
    fac=pd.merge(lev,tax,how='left',left_index=True,right_index=True)
    fac['CFLB%']=1/(1+(1/fac['lev ratio'])*(1/abs(1-fac['tax rate'])))*100
    fac['lev_unlev']=1+fac['lev ratio']*(1-fac['tax rate'])

    return fac

if __name__ =="__main__":
    fac,yl=prepare_hamada_yahoo("MSFT")

#==============================================================================
#==============================================================================












    


# -*- coding: utf-8 -*-


import os; os.chdir("S:/siat")
from siat.risk_evaluation import *

var,ratio=stock_VaR_normal_standard('BABA',1000,'2019-8-8',1,0.99)

tlcps=series_VaR_tlcp(['GOOG','MSFT','AAPL'],'2020-7-20',0.99,model='montecarlo')

from siat.var_model_validation import *
backtest_VaR(['AAPL'],[1000],'2020-7-20',1,model="normal_standard")


from siat.holding_risk import *

portfolio={'Market':('China','000001.SS'),'000661.SZ':2,'603392.SS':3,'300601.SZ':4}
vl,rl=get_VaR_portfolio(portfolio,'2020-7-20',1,0.99,model='all')



# -*- coding: utf-8 -*-
from siat import *

import os; os.chdir("S:/siat")
from siat.beta_adjustment import *

yearlist=gen_yearlist('2011','2020')
betas=betas_dji=get_beta_ML('EDU','^DJI',yearlist)

betas_dji=get_beta_ML('IDCBY','^DJI',yearlist)

betas_sp500=get_beta_ML('IDCBY','^GSPC',yearlist)

betas_sw=get_beta_SW('PG','^DJI', yearlist)

betas_sw_4452t=get_beta_SW('4452.T','^N225', yearlist)

yearlist=gen_yearlist('2011','2019')
betas_sw_hmif=get_beta_SW('HMI.F','^FCHI',yearlist)
betas_sw_diof=get_beta_SW('DIO.F','^FCHI', yearlist)

r=prepare_capm('600340.SS','000001.SS','2011-1-1','2020-12-31')
stock=get_price('600340.SS','2011-1-1','2020-12-31')
stock=get_price_ak_cn('600340.SS','2011-1-1','2020-12-31')

betas_dimson=get_beta_dimson('600376.SS','000001.SS', yearlist)
betas_dimson=get_beta_dimson('600340.SS','000001.SS', yearlist)

betas_hamada=get_beta_hamada2('600519.SS','000001.SS')

betas_hamada=get_beta_hamada2('600606.SS','000001.SS')

betas_hamada=get_beta_hamada2('GS','^GSPC')

#==============================================================================
stkcd='0700.HK'
mktidx='^HSI'
h=get_beta_hamada2(stkcd,mktidx)


from siat.financial_statements import *
fs_is=get_income_statements(stkcd).T
fs_bs=get_balance_sheet(stkcd).T

betas_hamada=get_beta_hamada_ts('600519.SS','000001.SS', yearlist)
import tushare as ts

pro=init_ts() 
is0=pro.income(ts_code='600519.SH')

token='49f134b05e668d288be43264639ac77821ab9938ff40d6013c0ed24f'
pro=ts.pro_api(token)
pro.income(ts_code='600519.sh')

R=prepare_capm(stkcd,mktidx,start,end)


betas1=get_beta_hamada2('0700.HK','^HSI')
betas1=get_beta_hamada2('MSFT','^GSPC')

betas1=get_beta_hamada2('BA','^GSPC')
betas1=get_beta_hamada2('GS','^GSPC')
betas1=get_beta_hamada2('AAPL','^GSPC')

betas1=get_beta_hamada2('000002.SZ','000001.SS')
betas1=get_beta_hamada2('600519.SS','000001.SS')
betas1=get_beta_hamada2('600606.SS','000001.SS')
#==============================================================================


# -*- coding: utf-8 -*-
"""
Created on Fri Sep 25 22:49:18 2020

@author: Peter
"""

import os; os.chdir("S:/siat")
from siat.financial_statements import *


bs=get_balance_sheet('0883.HK')
infot=info.T

info=get_balance_sheet('PDD')
infot=info.T
cr=compare_history(['BABA','PDD'],'Quick Ratio')

#==============================================================================
info=get_income_statements('MST')
info=get_income_statements('MDT')

info=get_cashflow_statements('MDT')
info=get_cashflow_statements('MST')

info=get_balance_sheet('MST')
info=get_balance_sheet('MDT')
info=get_balance_sheet('MSFT')

info=get_financial_statements('MDT')
info=get_financial_statements('MST')



# -*- coding: utf-8 -*-

import os; os.chdir("S:/siat")
from siat.option_pricing import *

df=stock_trend_by_option('JD')



c=bs_call(51100,51000,25,0.02828,0.1792)

datelist=option_maturity('MSFT')

dfc,dfp=option_chain('MSFT','2022-6-17')
dfc,dfp=option_chain('AAPL','2022-6-17')

datelist=option_maturity('^XSP')
dfc,dfp=option_chain('^XSP','2021-12-17')

datelist=option_maturity('TLT')
dfc,dfp=option_chain('TLT','2022-12-16')

df=stock_trend_by_option('JD')

from siat.transaction import *
df=security_price('JD','2020-12-1','2021-1-31')


bsm_aprice([30,50],42,183,0.015,0.23,90,1.5)
df=bsm_maturity(40,42,[200,100],0.015,0.23,90,1.5)
bsm_sigma(40,42,183,0.015,[0.1,0.4],90,1.5)

df=stock_trend_by_option('AAPL',7)
df=stock_trend_by_option('JD',7) 
df=stock_trend_by_option('MSFT',7) 
df=stock_trend_by_option('BABA',7) 
df=stock_trend_by_option('GS',7) 
df=stock_trend_by_option('PDD',7) 

df=recent_stock_split('AAPL')
df=recent_stock_split('MSFT')

get_last_close('AAPL')
get_last_close('MSFT')

import time
tupTime = time.localtime(1604350755)#秒时间戳
stadardTime = time.strftime("%Y-%m-%d %H:%M:%S", tupTime)
print(stadardTime)


ticker="AAPL"
mdate="2020-11-06"
opt_call, opt_put=option_chain(ticker,mdate,printout=False)


from siat.stock import *
div=stock_dividend('AAPL','2020-1-1','2020-11-3')
splt=stock_split('AAPL','2020-1-1','2020-11-3')


df=predict_stock_trend_by_option('AAPL')

df=predict_stock_trend_by_option('02020.HK',lastndays=7)
df=predict_stock_trend_by_option('2331.HK',lastndays=7)
df=predict_stock_trend_by_option('1368.HK',lastndays=7)

splt=stock_split('NKE','2020-1-1','2020-11-3')
df=predict_stock_trend_by_option('NKE',lastndays=30)

splt=stock_split('ADS.DE','2020-1-1','2020-11-3')
df=predict_stock_trend_by_option('ADS.DE',lastndays=30)

splt=stock_split('JD','2020-1-1','2020-11-3')
df=predict_stock_trend_by_option('JD')

splt=stock_split('BABA','2020-1-1','2020-11-3')
df=predict_stock_trend_by_option('BABA',lastndays=30)



# -*- coding: utf-8 -*-
"""
本模块功能：中国内盘和外盘期货
所属工具包：证券投资分析工具SIAT 
SIAT：Security Investment Analysis Tool
创建日期：2021年9月26日
最新修订日期：2021年9月28日
作者：王德宏 (WANG Dehong, Peter)
作者单位：北京外国语大学国际商学院
版权所有：王德宏
用途限制：仅限研究与教学使用，不可商用！商用需要额外授权。
特别声明：作者不对使用本工具进行证券投资导致的任何损益负责！
"""

#==============================================================================
#关闭所有警告
import warnings; warnings.filterwarnings('ignore')
from siat.common import *
from siat.grafix import *
#==========================================================================================
if __name__=='__main__':
    trade_date='2020-7-13'

def future_type_china_0(trade_date):
    """
    中国内盘期货：代码和品种的基础数据
    """
    #日期变换
    trade_date1=convert_date_ts(trade_date)
    
    import akshare as ak
    df=ak.futures_rule(trade_date=trade_date1)
    
    try:
        cols = df.columns.tolist()
    except:
        return None
    cols1 = cols[2:3] + cols[1:2] + cols[0:1] + cols[3:] # 将基金代码列名放前面
    
    #改变字段顺序
    df1=df[cols1]
    
    #去掉期权项目
    df2=df1[~df1['品种'].str.contains("期权")]  
    df2.sort_values(by=['代码'],ascending=True,inplace=True)
    df2.reset_index(inplace=True)
    del df2['index']
    
    return df2

#======================================================================================
def SBC2DBC(ustring):
    """
    字符串转换：半角转全角，便于打印对齐
    """
    rstring = ''
    for uchar in ustring:
        inside_code = ord(uchar)
        if inside_code == 0x0020:
            inside_code = 0x3000
        else:
            if not (0x0021 <= inside_code and inside_code <= 0x7e):
                rstring += uchar
                continue
        inside_code += 0xfee0
        rstring += chr(inside_code)
    return rstring
#======================================================================================
if __name__=='__main__':
    trade_date='2020-7-13'
    df=future_type_china_0(trade_date)    

def future_type_china_1(df):
    """
    列示中国内盘期货的全部代码和品种，每4个一行
    """
    print("\n===== 中国内盘期货代码品种一览表 =====\n")
    #遍历，合成一行
    itemlist=[]
    for row in df.itertuples():
        code=SBC2DBC(getattr(row,'代码'))
        name=SBC2DBC(getattr(row,'品种'))
        itemlist=itemlist+[code+'（'+name+'）']
    
    maxlen=0
    for i in itemlist:
        ilen=len(i)
        if maxlen < ilen: maxlen=ilen
    maxlen=maxlen+1
    
    rownum=0
    linetxt=''
    iteminline=3
    for i in itemlist:
        if rownum < iteminline:
            ifull=i+SBC2DBC('.')*(maxlen-len(i))
            linetxt=linetxt+ifull
            rownum=rownum+1
        else:
            print(linetxt)
            rownum=0; linetxt=''
        
    import datetime
    today = datetime.date.today()
    print('\n*** 数据来源：国泰君安,',today)
    
    return

if __name__=='__main__':
    future_type_china_1(df)     
#======================================================================================
if __name__=='__main__':
    code='BB2007'
    df=future_type_china_0(trade_date) 

def get_future_symbol(code):
    """
    获得一个具体期货合约的品种代码
    """
    code1=code.upper()
    
    letterlist=[chr(i).upper() for i in range(97,123)]
    symbol=''
    
    for ch in code1:
        if ch in letterlist:
            symbol=symbol+ch
        else:
            break
    
    return symbol
#======================================================================================

def future_type_china_2(df,code):
    """
    列示中国内盘期货的品种明细
    """
    symbol=get_future_symbol(code)
    
    df1=df[df['代码']==symbol]
    cols=list(df1)
    cols.pop()

    print("\n===== 中国内盘期货品种概况 =====\n")
    maxlen=0
    for c in cols:
        clen=hzlen(c)
        if maxlen < clen: maxlen=clen
    
    contract='合约'
    print(contract+' '*(maxlen-hzlen(contract))+':',code)
    for c in cols:
        print(c+' '*(maxlen-hzlen(c))+':',df1[c].values[0])

    import datetime
    today = datetime.date.today()
    print('\n数据来源：国泰君安,',today)
    
    return

if __name__=='__main__':
    future_type_china_2(df,'BB2007') 

#======================================================================================
if __name__=='__main__':
    code='BX2007'

def future_type_china(code=''):
    """
    综合程序：
    合约code为空或品种symbol不存在时列示中国内盘期货的全部代码，否则列示该合约品种明细
    """
    import datetime
    today = datetime.date.today()
    
    df=future_type_china_0(today)
    while df is None:
        today=date_adjust(today, adjust=-1)
        df=future_type_china_0(today)

    symbol=get_future_symbol(code)
    df1=df[df['代码']==symbol]  
    
    if len(df1)==0:
        if not (code==''):
            print("  Warning(future_type_china):",code,'not found')
        future_type_china_1(df)
        return df
    
    future_type_china_2(df,code)
    return df1

if __name__=='__main__':
    code='Bx2007'
    df=future_type_china(code) 

#======================================================================================
if __name__=='__main__':
    code='SC2110'
    start='2021-8-1'
    end='2021-9-27'
    power=0

def future_price_china(code,start,end,power=0):
    """
    综合程序：
    绘制内盘期货交易曲线：收盘价vs结算价vs成交量
    """
    print("Searching for contract "+code+", it costs great time, just take a break...")
    #日期变换
    start1=convert_date_ts(start)
    end1=convert_date_ts(end)
    
    #获得中国内盘期货品种代码
    varietydf=future_type_china_0(end1)
    #获得期货合约code的品种代码
    variety=get_future_symbol(code)
    vdf=varietydf[varietydf['代码']==variety]
    if len(vdf)==0:
        print("  #Error(future_price_china): future variety",variety,'not found')
        return None
    varietyname=vdf['品种'].values[0]
    
    #查找交易所代码
    mktname=vdf['交易所'].values[0]
    mktnamelist=['大商所','郑商所','上期所','中金所','能源中心']
    mktnamelistfull=['大连商品交易所','郑州商品交易所','上海期货交易所','中国金融期货交易所','上海国际能源交易中心']
    mktcodelist=['DCE','CZCE','SHFE','CFFEX','INE']
    pos=mktnamelist.index(mktname)
    market=mktcodelist[pos]
    mktnamefull=mktnamelistfull[pos]
    
    import akshare as ak
    p=ak.get_futures_daily(start_date=start1,end_date=end1,market=market)
    
    p1=p[p['symbol']==code]
    if len(p1)==0: 
        print("  #Error(future_price_china): future contract",code,'not found in market',market)
        contracts=set(list(p[p['variety']==variety]['symbol']))
        contracts1=sorted(contracts) 
        print("\n提示：当前可用的"+varietyname+variety+'期货合约：'+mktnamefull+', '+str(end))
        print(contracts1)
        return None        
    
    #转换日期格式
    import pandas as pd
    p1['date1']=pd.to_datetime(p1['date'])
    p2=p1.set_index('date1')
    
    import pandas as pd
    p2a=pd.DataFrame(p2['close']) #收盘价
    p2a['close']=p2a['close'].astype('float')
    
    p2b=pd.DataFrame(p2['settle']) #结算价  
    p2b['settle']=p2b['settle'].astype('float')
    
    p2c=pd.DataFrame(p2['volume']) #成交量
    p2c['volume']=p2c['volume'].astype('int64')
    
    p2d=pd.DataFrame(p2['turnover']) #成交金额
    p2d['turnover']=p2d['turnover'].astype('int64')
    
    p2e=pd.DataFrame(p2['open_interest']) #持仓量
    p2e['open_interest']=p2e['open_interest'].astype('int64')
    
    #绘图
    import datetime
    today = datetime.date.today()
    footnote="数据来源："+mktnamefull+', '+str(today)
    
    print("  Rendering trading trend graphics ...")
    titletxt="中国内盘期货交易走势分析："+varietyname+code
    #避免code被翻译
    acode=' '+code
    #"""
    #收盘价vs结算价
    plot_line2(p2a,acode,"close","收盘价", \
               p2b,acode,"settle","结算价", \
               '价格',titletxt,footnote, \
               power=power,twinx=False)
    #"""
    #收盘价vs成交量
    plot_line2(p2a,acode,"close","收盘价", \
               p2c,acode,"volume","成交量", \
               '',titletxt,footnote, \
               power=power,twinx=True)
    #"""
    #收盘价vs持仓量
    plot_line2(p2a,acode,"close","收盘价", \
               p2e,acode,"open_interest","持仓量", \
               '',titletxt,footnote, \
               power=power,twinx=True)
    #"""
    return p2

if __name__=='__main__':
    code='SC2110'
    df=future_type_china(code) 
    
#===========================================================================================
if __name__=='__main__':
    symbol="ZSD"

def future_type_foreign_0(symbol="ZSD"):
    """
    获得某个外盘期货品种symbol详情
    """
    import akshare as ak
    try:
        df=ak.futures_foreign_detail(symbol=symbol)
    except:
        print("  #Error(future_type_foreign_0): future variety",symbol,'not found')
        return None
    
    #构造df
    col1=df.iloc[0,0]; col1value=df.iloc[0,1]
    if "并非期货" in col1value:
        col1value=col1value.replace("并非期货",'')
    
    col2=df.iloc[0,2]; col2value=df.iloc[0,3]
    col3=df.iloc[0,4]; col3value=df.iloc[0,5]
    
    col4=df.iloc[1,0]; col4value=df.iloc[1,1]
    col5=df.iloc[1,2]; col5value=df.iloc[1,3]
    col6=df.iloc[1,4]; col6value=df.iloc[1,5]
    
    col7=df.iloc[2,0]; col7value=df.iloc[2,1]
    col8=df.iloc[2,2]; col8value=df.iloc[2,3]
    
    col9=df.iloc[3,0]; col9value=df.iloc[3,1]
    col10=df.iloc[3,2]; col10value=df.iloc[3,3]  
    
    future_dict={col9:col9value,col1:col1value,col10:col10value,col2:col2value, \
                 col3:col3value,col4:col4value,col5:col5value,col6:col6value, \
                     col7:col7value,col8:col8value}
    import pandas as pd
    df1=pd.DataFrame(future_dict,index=[0])
    
    return df1

if __name__=='__main__':
    df=future_type_foreign_0(symbol="ZSD")  
    print(df.T)
#===========================================================================================
if __name__=='__main__':
    symbol="ZSD"

def future_type_foreign_1():
    """
    获得中国所有外盘期货品种详情
    """
    import akshare as ak  
    varietylist=ak.futures_foreign_commodity_subscribe_exchange_symbol()
    varietylist.sort()
    
    try:
        df=future_type_foreign_0(symbol=varietylist[0])
    except:
        pass
    for v in varietylist:
        try:
            df_tmp=future_type_foreign_0(symbol=v)
            #print(v)
            df=df.append(df_tmp)
        except:
            pass
    df.drop_duplicates(inplace=True)
    
    return df
    
#===========================================================================================
if __name__=='__main__':
    symbol="ZSD"
    df=future_type_foreign_1()

def future_type_foreign_2(df,symbol="ZSD"):
    """
    打印中国外盘期货某个品种详情
    """
    
    df1=df[df['交易代码']==symbol]
    cols=list(df1)

    print("\n===== 中国外盘期货品种概况 =====\n")
    maxlen=0
    for c in cols:
        clen=hzlen(c)
        if maxlen < clen: maxlen=clen
    
    for c in cols:
        print(c+' '*(maxlen-hzlen(c))+':',df1[c].values[0])

    import datetime
    today = datetime.date.today()
    print('\n*** 数据来源：新浪财经,',today)
    
    return

if __name__=='__main__':
    future_type_foreign_2(df,symbol="ZSD")  
    
#======================================================================================
if __name__=='__main__':
    code='ZSD'

def future_type_foreign(code=''):
    """
    综合程序：
    合约code为空或不存在时列示中国外盘期货的所有品种，否则列示一个品种的明细条款
    """
    
    df=future_type_foreign_1()

    df1=df[df['交易代码']==code]  
    
    if len(df1)==0:
        if not (code==''):
            print("  Warning(future_type_foreign):",code,'not found')
        
        #设置打印对齐
        import pandas as pd
        pd.set_option('display.max_columns', 1000)
        pd.set_option('display.width', 1000)
        pd.set_option('display.max_colwidth', 1000)
        pd.set_option('display.unicode.ambiguous_as_wide', True)
        pd.set_option('display.unicode.east_asian_width', True)
        
        print("\n===== 中国外盘期货品种一览表 =====\n")
        df2=df[['交易代码','交易品种','上市交易所']]
        print(df2.to_string(index=False))
        
        import datetime
        today = datetime.date.today()
        print('\n*** 数据来源：新浪财经,',today)
        
        return df
    
    future_type_foreign_2(df,symbol=code)
    
    return df1

if __name__=='__main__':
    df=future_type_foreign(code='') 
    df=future_type_foreign(code='CT')
    df=future_type_foreign(code='OIL')

#====================================================================================== 
if __name__=='__main__':
    code='ZSD'
    start='2021-8-1'
    end='2021-9-27'
    power=0

def future_price_foreign(code,start,end,power=0):
    """
    综合程序：
    绘制中国外盘期货的交易趋势：收盘价vs成交量
    """
    print("Searching for contract "+code+", it costs great time, just take a break...")
    #日期变换
    flag,start1,end1=check_period(start,end)
    
    #获得中国外盘期货品种代码
    varietydf=future_type_foreign_0(symbol=code)
    #获得期货合约code的品种代码
    if len(varietydf)==0:
        print("  #Error(future_price_foreign): future variety",code,'not found')
        return None
    varietyname=varietydf['交易品种'].values[0]
    
    #查找交易所代码
    mktnamefull=varietydf['上市交易所'].values[0]
    
    import akshare as ak
    p0=ak.futures_foreign_hist(symbol=code) 
    p01=p0[p0['date']>=start1]
    p1=p01[p01['date']<=end1]
    
    if len(p1)==0: 
        print("  #Error(future_price_foreign): prices not found for",code,'from',start,'to',end)
        return None        
    
    #转换日期格式
    p2=p1.set_index('date')
    
    import pandas as pd
    p2a=pd.DataFrame(p2['close']) #收盘价
    p2a['close']=p2a['close'].astype('float')
    
    p2c=pd.DataFrame(p2['volume']) #成交量
    p2c['volume']=p2c['volume'].astype('int64')
    
    #绘图
    import datetime
    today = datetime.date.today()
    footnote="数据来源：新浪财经"+', '+str(today)
    
    titletxt="中国外盘期货交易走势分析："+varietyname+code
    
    #收盘价vs成交量
    #print("code =",code)
    #避免code被翻译
    acode=' '+code
    plot_line2(p2a,acode,"close","收盘价", \
               p2c,acode,"volume","成交量", \
               '',titletxt,footnote, \
               power=power,twinx=True)

    return p2

if __name__=='__main__':
    df=future_price_foreign('CT','2021-8-1','2021-9-28')
    
#===========================================================================================   


# -*- coding: utf-8 -*-


import os; os.chdir("S:\siat")
from siat.sector_china import *

df=sector_position_china('000661',"gn_swym")

df=sector_position_china('000661',"new_swzz")
df=sector_position_china('000661',"hangye_ZC27")




df=sector_list_china("新浪行业")



#数据网址：http://finance.sina.com.cn/stock/sl/#qmxindustry_1

import akshare as ak

# 板块行情
stock_industry_sina_df = ak.stock_sector_spot(indicator="新浪行业")
sectorlist=list(stock_industry_sina_df['板块'])
    num=len(sectorlist)
    for d in sectorlist:
        print(d,end='  ')
        pos=sectorlist.index(d)+1
        if (pos % 6 ==0) or (pos==num): print(' ')




stock_industry_star_df = ak.stock_sector_spot(indicator="启明星行业")

stock_industry_concept_df = ak.stock_sector_spot(indicator="概念")

stock_industry_region_df = ak.stock_sector_spot(indicator="地域")

stock_industry_industry_df = ak.stock_sector_spot(indicator="行业")
hangye_list=list(set(list(stock_industry_industry_df['label'])))
hangye_list.sort()

#板块详情：nmc-流通市值？mktcap-总市值？
#"行业"
stock_sector_zl01_df = ak.stock_sector_detail(sector="hangye_ZL01")
len(stock_sector_zl01_df)

stock_sector_zc27_df = ak.stock_sector_detail(sector="hangye_ZC27")

#"概念"
stock_sector_kc50_df = ak.stock_sector_detail(sector="gn_kc50")

#"地域"
stock_sector_440000_df = ak.stock_sector_detail(sector="diyu_440000")

#"新浪行业"
stock_sector_dlhy_df = ak.stock_sector_detail(sector="new_dlhy")

#"启明星行业"：无详情



import os; os.chdir("E:/siat")
from siat.sector_china import *

df=sector_list_china(indicator="概念")
sector_code_china("重组概念")
sector_code_china("建筑工程")
sector_code_china("资产注入")
sector_code_china("建筑节能")
sector_code_china("建筑装饰和其他建筑业")
sector_code_china("建筑安装业")






df=sector_rank_china("涨跌幅","概念",80)
df[df['板块']=='重组概念']
df1=sector_detail_china(sector="gn_zzgn",comp="涨跌幅",num=10)
df1=sector_detail_china(sector="gn_zzgn",comp="涨跌幅",num=-10)


df=sector_list_china(indicator="新浪行业")
df=sector_list_china(indicator="启明星行业")
df=sector_list_china(indicator="行业")
df=sector_list_china(indicator="概念")
df=sector_list_china(indicator="地域")









sector_code_china("资本市场服务")
sector_code_china("房地产")

df=sector_rank_china("涨跌幅","新浪行业")
df=sector_rank_china("涨跌幅","新浪行业",-10)

#df=sector_rank_china("成交量","新浪行业")
df=sector_rank_china("平均价格","新浪行业")
df=sector_rank_china("公司家数","新浪行业")
df=sector_rank_china("公司家数","新浪行业",-10)

df=sector_rank_china("涨跌幅","地域")
df=sector_rank_china("涨跌幅","地域",-10)

#df=sector_rank_china("成交量","地域")
df=sector_rank_china("平均价格","地域")

df=sector_rank_china("涨跌幅","启明星行业")
df=sector_rank_china("涨跌幅","启明星行业",-10)

df=sector_rank_china("涨跌幅","行业")
df=sector_rank_china("涨跌幅","行业",10)

df=sector_rank_china("涨跌幅","概念",-10)
df=sector_rank_china("公司家数","概念",-10)

df=sector_detail_china(sector="new_dlhy",comp="涨跌幅",num=-10)
df=sector_detail_china(sector="new_dlhy",comp="换手率",num=10)
df=sector_detail_china(sector="new_dlhy",comp="收盘价",num=10)
df=sector_detail_china(sector="new_dlhy",comp="市盈率",num=10)
df=sector_detail_china(sector="new_dlhy",comp="市净率",num=10)
df=sector_detail_china(sector="new_dlhy",comp="流通市值",num=10)
df=sector_detail_china(sector="new_dlhy",comp="总市值",num=10)
df=sector_detail_china(sector="new_dlhy",comp="流通市值",num=-10)


df=sector_position_china('600021',"new_dlhy")
df=sector_position_china('000661',"yysw")

df=sector_position_china('000661',"gn_swym")


df=sector_position_china('002504',"jzgc")
df=sector_position_china('002504',"gn_zczr")
df=sector_position_china('002504',"gn_jzjn")
df=sector_position_china('002504',"hangye_ZE50")

import seaborn as sn


